#include <math.h>
#include "mex.h"
#include "fitness_f.h"

#define I_C_IND prhs[0]
#define I_A_SUB prhs[1]
#define I_C_SUB_CLIENTS prhs[2]
#define I_A_CLIENT_BS prhs[3]
#define I_A_WEIGHT prhs[4]
#define O_DBL_FITNESS plhs[0]

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    USHORT nAtoms;
	char buf[1024];
	mxArray **ptr;

    /* Check for proper arguments */
	if (nrhs != 5)
		mexErrMsgTxt("Five input arguments are required.");
    else if (nlhs > 1)
		mexErrMsgTxt("Too many output arguments.");

	nAtoms = (USHORT)mxGetNumberOfElements(I_A_WEIGHT);

	if (!mxIsCell(I_C_IND) || mxGetN(I_C_IND) != 1)
		mexErrMsgTxt("1st input argument must be a N-by-1 cell array.");

	else if (!mxIsUint16(I_A_SUB) || mxGetM(I_A_SUB) != 2)
		mexErrMsgTxt("2nd input argument must be a 2-by-N array of UINT16.");

	else if (!mxIsCell(I_C_SUB_CLIENTS) || mxGetN(I_C_SUB_CLIENTS) != 1)
		mexErrMsgTxt("3rd input argument must be a subs-by-1 cell of UINT16 arrays.");

	else if (!mxIsUint32(I_A_CLIENT_BS) || mxGetM(I_A_CLIENT_BS) != (nAtoms/ULONG_BITS))
		{
			sprintf(buf, "4th input argument must be a %u-by-clients array of UINT32.", (nAtoms/ULONG_BITS));
			mexErrMsgTxt(buf);
		}

	else if (!mxIsDouble(I_A_WEIGHT) || mxGetNumberOfElements(I_A_WEIGHT) != nAtoms)
		{
			sprintf(buf, "5th input argument must be a double array with %u elements.", nAtoms);
			mexErrMsgTxt(buf);
		}

	/* Do computations */
	ptr = mxGetData(I_C_IND);
		
	if (mxIsCell(*ptr)) // Is population ?
	{
		int i;
		double *pfits;
		USHORT nPop = (USHORT)mxGetM(I_C_IND);
		O_DBL_FITNESS = mxCreateDoubleMatrix(1, nPop, mxREAL);
		pfits = mxGetPr(O_DBL_FITNESS);

		for (i = 0; i < nPop; ++i)
		{
			mxArray *pind = *ptr++;
			*pfits++ = fitness_f(nAtoms, mxGetM(pind), mxGetN(I_A_CLIENT_BS),
				mxGetData(pind), mxGetPr(I_A_SUB), mxGetPr(I_C_SUB_CLIENTS), 
				mxGetPr(I_A_CLIENT_BS), mxGetPr(I_A_WEIGHT));
		}
	}
	else if (mxIsUint16(*ptr)) // Is group data valid ?
		O_DBL_FITNESS = mxCreateScalarDouble(fitness_f(nAtoms, mxGetM(I_C_IND), mxGetN(I_A_CLIENT_BS),
				ptr, mxGetPr(I_A_SUB), mxGetPr(I_C_SUB_CLIENTS), 
				mxGetPr(I_A_CLIENT_BS), mxGetPr(I_A_WEIGHT)));

    return;
}
