% (C) Mario Guimaraes 4 Nov 2001

function [aWeights, asubjects, asubscriptions, aclients, aclientsubs, aclient_bin_subs, nsubs] = gen_subdata()

global nclients nsubjects nsbjcategories psubjects natoms nminsubspercli nmaxsubspercli

persistent nmaxsub;

nsubid = 0;

nsbjpercategory = fix(nsubjects / nsbjcategories);
nmaxsbjpercli = min(fix(nsubjects * psubjects), nsbjpercategory);
nmaxsubpercli = nmaxsubspercli - nminsubspercli + 1;
if isempty(nmaxsub), nmaxsub = nmaxsbjpercli * nmaxsubpercli * nclients; end

% Generate atomic volumes for each subject
aWeights = gen_atoms;

% Generate client subscriptions
asubjects = cell(nsubjects, 1); % for each subject stores a list of subscription IDs
atmpSubscriptions = uint16(zeros(2, nmaxsub)); % for each subscription stores the pair (atom_i,atom_f)
atmpClients = cell(nmaxsub, 1); % for each subscription stores the list of its clients
atmpClientSubs = cell(nclients, 1); % for each client stores the list of its subscriptions

for icli = 1:nclients
    % choose subjects to subscribe
    csubs = gen_sub;
    ncnt = size(csubs,1);
    
    % for each subject
    while ncnt > 0
        sbj = csubs{ncnt,1};
        sout = csubs{ncnt,2};
        asjb_subIDs = asubjects{sbj};

        if isempty(asjb_subIDs)
            anew = sout;
        else            
            anew = [];
            asjb_subs = atmpSubscriptions(:,asjb_subIDs);
            
            for sub = sout
                aidx = find(sub(1) == asjb_subs(1,:) & sub(2) == asjb_subs(2,:));                
                
                if isempty(aidx)
                   anew = [anew sub];
                else % add client to an existing subscription
                    sid = asjb_subIDs(aidx);
                    atmpClients{double(sid)} = [atmpClients{double(sid)} uint16(icli)];
                    atmpClientSubs{icli} = [atmpClientSubs{icli} sid];
                end
            end
        end

        if ~isempty(anew)
            nnew = size(anew,2);
            anew_subIDs = nsubid + [1:nnew];
            asubjects{sbj} = [asubjects{sbj} anew_subIDs];
            atmpClients(anew_subIDs) = {uint16(icli)};
            atmpClientSubs{icli} = [atmpClientSubs{icli} anew_subIDs];
            atmpSubscriptions(:,anew_subIDs) = anew;
            nsubid = nsubid + nnew;
        end
        
        ncnt = ncnt - 1;
    end
end

nmaxsub = nsubid;
nsubs = nmaxsub;

% Generate subscription data for each client in binary form.
nmax_atom_id = natoms .* nsubjects;
aclient_bin_subs = uint32(zeros(nmax_atom_id ./ 32, nclients));
    
for i = 1:nclients
    abits = union_bs(atmpSubscriptions(:,atmpClientSubs{i}), nmax_atom_id);
    aclient_bin_subs(:,i) = bitxor(abits, 2^32-1);
end

% Shuffle subscriptions IDs.
    %Since we create many sucessive subscriptions
    % for a subject, it is convenient to assign non consecutive IDs to those subscriptions so the genetic algorithm
    % generates a diverse population (remember that in each group we cross by subscription ID).
    % Also, in real world the IDs of a subject's subscriptions are non consecutive.
atmpSubIDs = uint16(randperm(nmaxsub));
asubscriptions = uint16(zeros(2, nmaxsub)); % for each subscription stores the pair (atom_i,atom_f)
aclients = cell(nmaxsub, 1); % for each subscription stores the list of its clients
aclientsubs = cell(nclients, 1); % for each client stores the list of its subscriptions

for sbj = 1:nsubjects
    asjb_subIDs = asubjects{sbj}; 
    asubjects{sbj} = atmpSubIDs(asjb_subIDs);
    asubscriptions(:,asubjects{sbj}) = atmpSubscriptions(:,asjb_subIDs);
    aclients(asubjects{sbj}) = atmpClients(asjb_subIDs);
end

for cid = 1:nclients
    aclientsubs{cid} = atmpSubIDs(atmpClientSubs{cid});
end